<?PHP
/**
 * Functions for communication with DHL-IntraShip Webservice
 * Version 1.0		Mac 2013-05-29
 * Supports only DHL-Paket, DHL-Europaket, DHL-Weltpaket
 * COD DHL-Paket only
 */

/**
 * Connect DHL and retreive Package / Licenseplate Numbers for a shipment
 * @param string	$ShipmentArray	Array with Sender / Shipment Data
 * @param string	$PackageData	Packaging / Weight Data for each package in this shipment
 * @param boolean	$Debug	Default False, if True request and response will be saved as "request.txt" and "response.txt"
 * @return string with XML-Answer or "ERROR" plus some debug information
 */
Function CreateShipment($ShipmentArray,$PackageData,$Debug=False)
{
$NumPacks = count($PackageData);
$XML=MakeSomeXML("",0,"soap:Envelope xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\" xmlns:cis=\"http://dhl.de/webservice/cisbase\" xmlns:de=\"http://de.ws.intraship\"","");
$XML=MakeSomeXML($XML,1,"soap:Header");
$XML=MakeSomeXML($XML,2,"cis:Authentification");
$XML=MakeSomeXML($XML,3,"cis:user",$ShipmentArray['User']);						// Our user name
$XML=MakeSomeXML($XML,3,"cis:signature",$ShipmentArray['Signature']);			// Our signature
$XML=MakeSomeXML($XML,3,"cis:type","0");										// DHL wants this...
// We don't send any of the "DeveloperAuthentification", "DEVID", "APPID" stuff. It's optional, so why bother the server with it?
$XML=MakeSomeXML($XML,2,"/cis:Authentification");
$XML=MakeSomeXML($XML,1,"/soap:Header");										// Header finished, start with the body
$XML=MakeSomeXML($XML,1,"soap:Body");
$XML=MakeSomeXML($XML,2,"de:CreateShipmentDDRequest");
$XML=MakeSomeXML($XML,3,"cis:Version");
$XML=MakeSomeXML($XML,4,"cis:majorRelease","1");								// Software Version 1.0 required
$XML=MakeSomeXML($XML,4,"cis:minorRelease","0");
$XML=MakeSomeXML($XML,3,"/cis:Version");
$XML=MakeSomeXML($XML,3,"ShipmentOrder");
$XML=MakeSomeXML($XML,4,"SequenceNumber","1");									// Only one request at a time
$XML=MakeSomeXML($XML,4,"Shipment");
$XML=MakeSomeXML($XML,5,"ShipmentDetails");
$XML=MakeSomeXML($XML,6,"ProductCode",$ShipmentArray['ShipType']);				// DHL Product code EPN (Paket)/ EPI (Europaket) / BPI (Weltpaket)
$XML=MakeSomeXML($XML,6,"ShipmentDate",$ShipmentArray['ShipDate']);
// We don't declare value of goods
$XML=MakeSomeXML($XML,6,"cis:EKP",$ShipmentArray['DHLAccount']);				// 10 digit main account number
$XML=MakeSomeXML($XML,6,"Attendance");
$XML=MakeSomeXML($XML,7,"cis:partnerID",$ShipmentArray['DHLPartnerID']);		// last two digits of full DHL-Account #
$XML=MakeSomeXML($XML,6,"/Attendance");
$XML=MakeSomeXML($XML,6,"CustomerReference",$ShipmentArray['Reference']);		// Shipment reference

for ($i=0 ; $i < $NumPacks; $i++)
	{
	$XML=MakeSomeXML($XML,6,"ShipmentItem");
	$XML=MakeSomeXML($XML,7,"WeightInKG",$PackageData[$i]['Weight']);
	$XML=MakeSomeXML($XML,7,"PackageType",$PackageData[$i]['Package']);
	$XML=MakeSomeXML($XML,6,"/ShipmentItem");
	}

if ($ShipmentArray['COD'] != "") // We need some cash
	{
	$XML=MakeSomeXML($XML,6,"Service");
	$XML=MakeSomeXML($XML,7,"ServiceGroupOther");
	$XML=MakeSomeXML($XML,8,"COD");
	$XML=MakeSomeXML($XML,9,"CODAmount",$ShipmentArray['COD']);
	$XML=MakeSomeXML($XML,9,"CODCurrency","EUR");
	$XML=MakeSomeXML($XML,8,"/COD");
	$XML=MakeSomeXML($XML,7,"/ServiceGroupOther");
	$XML=MakeSomeXML($XML,6,"/Service");
	}
$XML=MakeSomeXML($XML,6,"Service");
$XML=MakeSomeXML($XML,7,"ServiceGroupDHLPaket");
	
if ($NumPacks > 1)	$XML=MakeSomeXML($XML,8,"Multipack","True");
	else $XML=MakeSomeXML($XML,8,"Multipack","False");
	
$XML=MakeSomeXML($XML,7,"/ServiceGroupDHLPaket");
$XML=MakeSomeXML($XML,6,"/Service");

if ($ShipmentArray['COD'] != "") // COD needs bank data
	{
	$XML=MakeSomeXML($XML,6,"BankData");
	$XML=MakeSomeXML($XML,7,"cis:accountOwner",$ShipmentArray['AccOwner']);
	$XML=MakeSomeXML($XML,7,"cis:accountNumber",$ShipmentArray['AccNumber']);
	$XML=MakeSomeXML($XML,7,"cis:bankCode",$ShipmentArray['BLZ']);
	$XML=MakeSomeXML($XML,7,"cis:bankName",$ShipmentArray['Bankname']);
	$XML=MakeSomeXML($XML,7,"cis:iban",$ShipmentArray['IBAN']);
	$XML=MakeSomeXML($XML,7,"cis:note",$ShipmentArray['Reference']);
	$XML=MakeSomeXML($XML,7,"cis:bic",$ShipmentArray['BIC']);					// BIC must be 8 or 11 chars. IMPORTANT!
	$XML=MakeSomeXML($XML,6,"/BankData");
	}

$XML=MakeSomeXML($XML,5,"/ShipmentDetails");

$XML=MakeSomeXML($XML,5,"Shipper");
$XML=MakeSomeXML($XML,6,"Company");
$XML=MakeSomeXML($XML,6,"cis:Company");
$XML=MakeSomeXML($XML,7,"cis:name1",$ShipmentArray['Absender']);
$XML=MakeSomeXML($XML,6,"/cis:Company");
$XML=MakeSomeXML($XML,6,"/Company");
$XML=MakeSomeXML($XML,6,"Address");
$XML=MakeSomeXML($XML,7,"cis:streetName",$ShipmentArray['AbStrasse']);
$XML=MakeSomeXML($XML,7,"cis:streetNumber",$ShipmentArray['AbHausnr']);
$XML=MakeSomeXML($XML,7,"cis:Zip");
$XML=MakeSomeXML($XML,8,"cis:germany",$ShipmentArray['AbPLZ']);
$XML=MakeSomeXML($XML,7,"/cis:Zip");
$XML=MakeSomeXML($XML,7,"cis:city",$ShipmentArray['AbStadt']);
$XML=MakeSomeXML($XML,7,"cis:Origin");
$XML=MakeSomeXML($XML,8,"cis:countryISOCode","DE");
$XML=MakeSomeXML($XML,7,"/cis:Origin");
$XML=MakeSomeXML($XML,6,"/Address");
$XML=MakeSomeXML($XML,6,"Communication");									// We ignore the communication fields
$XML=MakeSomeXML($XML,7,"cis:phone","1234567");
$XML=MakeSomeXML($XML,7,"cis:email","me@here.de");
$XML=MakeSomeXML($XML,7,"cis:contactPerson","Mr. Tester");
$XML=MakeSomeXML($XML,6,"/Communication");
$XML=MakeSomeXML($XML,5,"/Shipper");

$XML=MakeSomeXML($XML,5,"Receiver");
$XML=MakeSomeXML($XML,6,"Company");
$XML=MakeSomeXML($XML,7,"cis:Company");
$XML=MakeSomeXML($XML,8,"cis:name1",$ShipmentArray['EmpfLine1']);
if ($ShipmentArray['EmpfLine2'] != "") $XML=MakeSomeXML($XML,8,"cis:name2",$ShipmentArray['EmpfLine2']);
$XML=MakeSomeXML($XML,7,"/cis:Company");
$XML=MakeSomeXML($XML,6,"/Company");
$XML=MakeSomeXML($XML,6,"Address");
$XML=MakeSomeXML($XML,7,"cis:streetName",$ShipmentArray['EmpfStrasse']);
$XML=MakeSomeXML($XML,7,"cis:streetNumber",$ShipmentArray['EmpfHausnr']);
$XML=MakeSomeXML($XML,7,"cis:Zip");
$XML=MakeSomeXML($XML,8,"cis:other",$ShipmentArray['EmpfPLZ']);
$XML=MakeSomeXML($XML,7,"/cis:Zip");
$XML=MakeSomeXML($XML,7,"cis:city",$ShipmentArray['EmpfStadt']);
$XML=MakeSomeXML($XML,7,"cis:Origin");
$XML=MakeSomeXML($XML,8,"cis:countryISOCode",$ShipmentArray['EmpfLand']);
$XML=MakeSomeXML($XML,7,"/cis:Origin");
$XML=MakeSomeXML($XML,6,"/Address");
$XML=MakeSomeXML($XML,6,"Communication");
$XML=MakeSomeXML($XML,7,"cis:phone",$ShipmentArray['EmpfTel']);				// Euro / Weltpaket needs receiver contact phone
$XML=MakeSomeXML($XML,7,"cis:contactPerson",$ShipmentArray['EmpfLine1']);	// Euro / Weltpaket needs also contact person - we use adress line 1 here, because we create our own labels
$XML=MakeSomeXML($XML,6,"/Communication");
$XML=MakeSomeXML($XML,5,"/Receiver");

$XML=MakeSomeXML($XML,4,"/Shipment");
$XML=MakeSomeXML($XML,4,"LabelResponseType",$ShipmentArray['Request']);		// XML or URL
$XML=MakeSomeXML($XML,3,"/ShipmentOrder");
$XML=MakeSomeXML($XML,2,"/de:CreateShipmentDDRequest");
$XML=MakeSomeXML($XML,1,"/soap:Body");
$XML=MakeSomeXML($XML,0,"/soap:Envelope");

if ($Debug === True) file_put_contents("request.txt",$XML);
// $XML=file_get_contents("istest.txt");
$Response=DoSoapRequest("http://test-intraship.dhl.com/ws/1_0/de/ISService","CreateShipmentDD",$XML);
$Response = str_replace("&#xd;","\r\n",$Response);	// Insert some pretty CR/LF for our Windows-Editor-friends
$Response = str_replace("&lt;","<",$Response);		// and some brackets make it even more readable
if ($Debug === True) file_put_contents("response.txt",$Response);
return $Response;
}

/**
 * Very stupid function to create a bright and shiny XML-thingy
 * @param string	$XML	Actual XML-String
 * @param string	$Level	Number of Tabs to add 0-n
 * @param string	$Key	XML-Key
 * @param string	$Value	optional value. For an open key, just use "", otherwise return will be <key>value</key>
 * @return XML string with added line
 */
Function MakeSomeXML($XML,$Level,$Key,$Value="")
{
$Entry=str_pad("",$Level ,"\t");	// Do some tab-inserts to make it pretty
$Entry .= "<$Key>" . $Value;
if ($Value != "") $Entry .= "</$Key>";
$Entry .= "\r\n";
return $XML . $Entry;
}

/**
 * Create a SOAP-Request and return either XML-Response or "ERROR" plus debug informations
 * @param string	$URL	URL to connect
 * @param string	$Request	XML-Request
 * @return XML-Response or string containing "ERROR" with info if something went on the fritz
 */

Function DoSoapRequest($URL,$Action,$Request)
{
$header = array(
	"Content-Type: application/soap+xml;charset=iso-8859-1",
	"Accept: text/xml;charset=iso-8859-1",
	"Cache-Control: no-cache",
	"Pragma: no-cache",
	"Content-Length: ".strlen($Request),
	"SOAPAction: \"$Action\""
	);

$soap_do = curl_init();
curl_setopt($soap_do, CURLOPT_URL, $URL );
curl_setopt($soap_do, CURLOPT_CONNECTTIMEOUT, 30);		// Better 30 secs, DHL is an acronym for "dauert halt lnger"
curl_setopt($soap_do, CURLOPT_TIMEOUT,        30);
curl_setopt($soap_do, CURLOPT_RETURNTRANSFER, true );
curl_setopt($soap_do, CURLOPT_SSL_VERIFYPEER, false);
curl_setopt($soap_do, CURLOPT_SSL_VERIFYHOST, false);
curl_setopt($soap_do, CURLOPT_POST,           true );
curl_setopt($soap_do, CURLOPT_POSTFIELDS,     $Request);
curl_setopt($soap_do, CURLOPT_HTTPHEADER,     $header);

$XML=curl_exec($soap_do);
$info = curl_getinfo($soap_do);
curl_close($soap_do);
if ($XML === false || $info['http_code'] != 200) return "ERROR\n" . print_r($info,true) . $XML;
return $XML;
}
?>